// Example sketch showing how to create a node thay repeates messages
// from nodes far from gateway back to gateway. 
// It is important that nodes that has enabled repeater mode calls  
// gw.preocess() frequently. This node should never sleep. 

#include <MySensor.h>
#include <SPI.h>
#include <EEPROM.h>
#include <RCSwitch.h>

#define CHILD_ID 1 // Id of the sensor child
#define CHILD_ID_DEV 254 

unsigned long SEND_FREQUENCY = 30000; // Minimum time between send (in milliseconds). We don't wnat to spam the gateway. 
unsigned long RECEIVE_TIMEOUT = 500;

#define RECEIVEPIN 0  // 2 Pin Interupt
#define SENDPID 4

#define SendTry 3

//===============================================
MySensor gw;
RCSwitch mySwitch = RCSwitch();

unsigned long currentTime; 
unsigned long lastSend;
unsigned long lastRCSend;
unsigned long lastRCId;
unsigned long SendRCId = 0;

MyMessage RCMsg(CHILD_ID, V_IR_RECEIVE);
MyMessage DevMsg(CHILD_ID_DEV, V_VAR5);

void setup()  
{  
  wdt_disable();
  
  // Init adress from pin
  if (analogRead(3) >= 1023){
    Serial.begin(115200);
    Serial.println("Init adress");
    
    for (int i=0;i<512;i++)
      EEPROM.write(i, 0xFF);    
  }  
  
  // The third argument enables repeater mode.
  gw.begin(incomingMessage, 6, true);

  //Send the sensor node sketch version information to the gateway
  gw.sendSketchInfo("Repeater Node", "1.0");

  gw.present(CHILD_ID, S_ARDUINO_NODE, "RC433");  
  gw.present(CHILD_ID_DEV, S_ARDUINO_NODE, "Arduino sensors"); 

  //=== RC
  mySwitch.enableReceive( RECEIVEPIN );
  mySwitch.enableTransmit( SENDPID );    
  
  wdt_enable(WDTO_8S);  
  
  lastSend=millis();
  lastRCSend=0;
  lastRCId = 0;
}

void loop() 
{
  wdt_reset();

  currentTime = millis();

  //==== RC send
  if (SendRCId != 0)
  {
    for (int i=0;i<SendTry;i++){
      mySwitch.send( SendRCId, 24 );
      //gw.sleep(50);
    }

    Serial.print("Send RC ");
    Serial.println(SendRCId);
    
    SendRCId = 0;
  } 
  
  //==== RC receive
  if (mySwitch.available()) {    
    Serial.print("Receive ");
    Serial.print(mySwitch.getReceivedValue());
    Serial.print("-");
    Serial.print(mySwitch.getReceivedBitlength());
    Serial.print("bit-P");
    Serial.println(mySwitch.getReceivedProtocol());    

    if ((mySwitch.getReceivedBitlength() == 24) && (mySwitch.getReceivedProtocol() == 1))
    {
      unsigned long id = mySwitch.getReceivedValue();
      if ((currentTime - lastRCSend > RECEIVE_TIMEOUT) || (lastRCId != id))
        if (gw.send(RCMsg.set(id), true))
        {
          lastRCSend = currentTime;        
          lastRCId = id;
        }
    }
    
    mySwitch.resetAvailable();
  }  
  
  //=== Repeater
  // Send feq
  // Only send values at a maximum frequency or woken up from sleep  
  if (currentTime - lastSend > SEND_FREQUENCY)
    if (gw.send(DevMsg.set("Pong")))
      lastSend=currentTime; 
  
  // By calling process() you route messages in the background
  gw.process();
}

void incomingMessage(const MyMessage &message)
{ 
  if (message.isAck()) return;

  if (message.sensor != CHILD_ID) return;
  if (message.type != V_IR_SEND) return;

  // Set to task
  SendRCId = message.getLong();
}

